# Copyright (C) 2000-2001 The OpenRPG Project
#
#    openrpg-dev@lists.sourceforge.net
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# --
#
# File: mapper/images.py
# Author: OpenRPG
# Maintainer:
# Version:
#   $Id: images.py,v 1.21 2007/12/11 04:07:15 digitalxero Exp $
#
# Description:
#
__version__ = "$Id: images.py,v 1.21 2007/12/11 04:07:15 digitalxero Exp $"

import urllib
import Queue
import thread
import string
from threading import Lock
import mimetypes

from orpg.orpg_windows import *
#import gdic
import orpg.dirpath
import time

## Image cache globals
# Default image cache size -- may be over-ridden
IMAGE_CACHE_SIZE = 32
imageCache = {}
imageQueue = Queue.Queue(0)
imageFetchList = {}
imageFetchLock = Lock()


def load_img(path, object_type, image_object):
    """This funciton loads an image from a URL and returns a wx.Bitmap"""

    # If we already have it cached, simply return the image to the caller
    if imageCache.has_key(path):
        return imageCache[path]


    # Now, keep track of which URI we are fetching to optimize
    # the applications of the fetched images
    imageFetchLock.acquire()
    if not imageFetchList.has_key(path):
        imageFetchList[path] = [path, None, -1, object_type]

        # Now, start up a thread to asynchronously fetch our resource via it's URI
        thread.start_new_thread(thread_get_data, (path, object_type, image_object))
    else:
        thread.start_new_thread(thread_get_data, (path, object_type, image_object, True))
    imageFetchLock.release()

    img = wx.Bitmap(orpg.dirpath.dir_struct["icon"] + "fetching.png", wx.BITMAP_TYPE_PNG)
    return img


def thread_get_data(path, object_type, image_object, cache=False):
    """Thread which fetches the url passed in as path."""

    # Make sure the path is valid and generally safe to references an URI
    if imageFetchList[path] is False:
        print "This path does not refer to a valid image, if you have corrected the issue you need to clear the image cache so the system will check again"
        return

    # Don't quote colons and slashes
    if cache:
        try:
            while imageFetchList[path][1] == None:
                time.sleep(0.025)
        except:
            print "This path does not refer to a valid image, if you have corrected the issue you need to clear the image cache so the system will check again"
            return

        imageFetchLock.acquire()
        try:
            (path, img, imgType, object_type) = imageFetchList[path]
            imageQueue.put((path, img, imgType, object_type, image_object))
        finally:
            imageFetchLock.release()
        return

    #uriPath = urllib.quote(path, ":/")
    uriPath = urllib.unquote(path)

    imageFetchLock.acquire()
    try:
        d = urllib.urlretrieve(uriPath)
        # We have to make sure that not only did we fetch something, but that

        # it was an image that we got back.
        if d[0] and d[1].getmaintype() == "image":
            if imageFetchList.has_key(path):
                imageFetchList[path][1] = d[0]
                imageFetchList[path][2] = d[1].gettype()
                imageQueue.put((path, d[0], d[1].gettype(), object_type, image_object))
            else:
                imageQueue.put((path, d[0], d[1].gettype(), object_type, image_object))

        else:
            print "Image refused to load or URI did not reference a valid image: " + path
            imageFetchList[path] = False

    except IOError:
        imageFetchList[path] = False
        print "Unable to resolve/open the specified URI; image was NOT laoded:" + path
    finally:
        imageFetchLock.release()


def flushImageCache():
    """This function will flush all images contained within the image cache.
Plese note that this function must be called from the primary thread as it is not thread safe.  As such, it assumes that nothing else is working on the list at the same time this function is working"""
    imageFetchLock.acquire()
    try:
        keyList = imageCache.keys()
        for key in keyList:
            del imageCache[key]

        keyList = imageFetchList.keys()
        for key in keyList:
            del imageFetchList[key]
    finally:
        imageFetchLock.release()

    urllib.urlcleanup()
