import wx
import wx.html
import webbrowser
import urllib
import zipfile
import traceback
import md5
from orpg.orpgCore import *
import orpg.orpg_version
import orpg.tools.orpg_log
import orpg.orpg_xml
import orpg.dirpath
import orpg.tools.orpg_settings
import orpg.tools.validate
import orpg.tools.updaterwizard
import orpg.orpg_version

class orpgUpdater(orpg.tools.updaterwizard.AutoUpdater):
    def __init__(self, *args, **kwargs):
        orpg.tools.updaterwizard.AutoUpdater.__init__(self, *args, **kwargs)

    def Finished(self):
        self.settings.set_setting("PackagesType", self.PackageType)
        self.settings.set_setting("PackagesName", self.PackageName)
        self.settings.save()


class updateApp(wx.App):
    def OnInit(self):
        self.openrpg = open_rpg({})
        self.log = orpg.tools.orpg_log.orpgLog(orpg.dirpath.dir_struct["user"] + "runlogs/")
        self.log.setLogToConsol(False)
        self.log.log("Updater Start", ORPG_NOTE)

        #Add the initial global components of the openrpg class
        #Every class should be passed openrpg
        self.openrpg.add_component("log", self.log)
        self.openrpg.add_component("xml", orpg.orpg_xml)

        self.validate = orpg.tools.validate.Validate()
        self.openrpg.add_component("validate", self.validate)

        self.settings = orpg.tools.orpg_settings.orpgSettings(self.openrpg)
        self.openrpg.add_component("settings", self.settings)

        self.settings.updateIni()

        self.updater = orpgUpdater(None, wx.ID_ANY, "OpenRPG Patcher") #updaterFrame(self, "OpenRPG Patcher", self.openrpg)
        self.updater.Url = 'http://openrpg.digitalxero.net/'
        self.updater.UpdateFile = 'updates/package_list.xml'
        self.updater.Rollback = True
        self.updater.Version = orpg.orpg_version.VERSION
        self.updater.Build = orpg.orpg_version.BUILD
        self.updater.PackageDirectory = orpg.dirpath.dir_struct["home"]
        self.updater.PackageType = self.settings.get_setting("PackagesType")
        self.updater.PackageName = self.settings.get_setting("PackagesName")
        self.updater.settings = self.settings
        self.updater.Start()

        return True

    def OnExit(self):
        self.settings.save()
        imported = ['orpg.orpgCore', 'orpg.orpg_wx', 'orpg.orpg_version', 'orpg.tools.orpg_log', 'orpg.orpg_xml', 'orpg.dirpath', 'orpg.tools.orpg_settings', 'orpg.tools.validate', 'orpg.pulldom', 'orpg.tools.NotebookCtrl', 'orpg.tools.config_update', 'orpg.systempath', 'orpg.minidom', 'orpg.dirpath.dirpath_tools', 'orpg.tools.rgbhex', 'orpg.orpg_windows', 'orpg.tools.updaterwizard']

        for name in imported:
            if name in sys.modules:
                self.log.log("Unimported " + name, ORPG_DEBUG)
                del sys.modules[name]

        self.log.log("Updater Exit\n\n", ORPG_NOTE)