import wx
import wx.html
import wx.wizard
from string import *
import os
import webbrowser
import urllib
import traceback
import md5
import xml.dom.minidom as minidom


class NotesHTMLWindow(wx.html.HtmlWindow):
    """Window used to display the Release Notes"""
    def __init__( self, parent):
        wx.html.HtmlWindow.__init__(self, parent, wx.ID_ANY)

    def OnLinkClicked( self, ref ):
        """Open an external browser to resolve links!!!"""
        href = ref.GetHref()
        webbrowser.open( href )

class FirstPage(wx.wizard.PyWizardPage):
    def __init__(self, parent, *args, **kwargs):
        wx.wizard.PyWizardPage.__init__(self, parent)

        self.parent = parent

        self._next = None
        self._prev = None

        self._noteswnd = NotesHTMLWindow(self)
        self._notes = None
        self._msgwnd = wx.TextCtrl(self, wx.ID_ANY, style=wx.TE_MULTILINE | wx.TE_READONLY)
        self._updateList = {}
        self._pkgbtn = wx.Button(self, wx.ID_ANY, "Package Select")
        self._progressbar = wx.Gauge(self, wx.ID_ANY, 100)
        self._notConected = wx.StaticText(self, wx.ID_ANY, "Could not Read update faile, either you are not connected to the internet or the update site is unavaliable at this time")
        self._done = wx.StaticText(self, wx.ID_ANY, "There are no new updates avaliable. Please Click Finish.")

        self.Bind(wx.EVT_BUTTON, self._switchpkg, self._pkgbtn)

        self._sizer = wx.BoxSizer(wx.VERTICAL)
        self._pkgsizer = wx.BoxSizer(wx.VERTICAL)
        self._notesizer = wx.BoxSizer(wx.HORIZONTAL)

        self._sizer.Add(self._progressbar, 0, wx.EXPAND)
        self._sizer.Add(self._notConected, 0, wx.EXPAND)
        self._sizer.Add(self._done, 0, wx.EXPAND)

        self._notesizer.Add(self._noteswnd, 1, wx.EXPAND)
        self._notesizer.Add(self._msgwnd, 1, wx.EXPAND)

        self._sizer.Add(self._notesizer, 1, wx.EXPAND)
        self._sizer.Add(self._pkgbtn, 0, wx.EXPAND)

        self._hideAll()

        self._pkgs = {}

        self._lastPage = ''

        self.parent.DisableButton(wx.ID_CANCEL)
        self.parent.DisableButton(wx.ID_FORWARD)

        try:
            self.xml_dom = minidom.parse(urllib.urlopen(self.parent.Url + self.parent.UpdateFile))
        except:
            self.xml_dom = None
        urllib.urlcleanup()

        self.SetSizer(self._sizer)
        self.SetAutoLayout(True)
        self.Layout()

        self.timer = wx.Timer(self, wx.NewId())
        self.Bind(wx.EVT_TIMER, self._fixNext)
        self.timer.Start(250)

        self.parent.Bind(wx.wizard.EVT_WIZARD_PAGE_CHANGING, self._endTimer)

        if self.xml_dom is None:
            self._showNoUpdate(False)
        else:
            self._checkUpdate()

    def _fixNext(self, evt):
        self.parent.FindWindowById(wx.ID_FORWARD).SetLabel("Update")

    def _endTimer(self, evt):
        self._hideAll()
        self.timer.Stop()
        self.parent.FindWindowById(wx.ID_FORWARD).SetLabel("Finish")
        try:
            self.Next.Start()
            self.Next = None
        except:
            pass

    def _checkUpdate(self, updateAvaliable=False):
        """
            Option1: No update avaliable, thus no next page
            Option2: This package no longer availiage, This page show package select (recheck update once one is selected)
            Option3: Update avaliable, This page shows Release Notes, next page = Download update
            Option4: Not connected to internet or update page is down, no next page
        """
        self._hideAll()
        self._progressbar.Show()
        self.Layout()

        #Build self._pkgs
        if len(self._pkgs) == 0:
            ptypes = self.xml_dom.getElementsByTagName('package_type')
            self._progressbar.SetRange(len(ptypes))
            i = 0
            for ptype in ptypes:
                wx.Yield()
                self._progressbar.SetValue(i)
                i += 1
                self._pkgs[ptype.getAttribute('name')] = {}

                pkgs = ptype.getElementsByTagName('orpg_package')

                for pkg in pkgs:
                    self._pkgs[ptype.getAttribute('name')][pkg.getAttribute('name')] = pkg

        #Must have a packageType and Name inorder to check the Update
        if self.parent.PackageType is None or self.parent.PackageName is None:
            self._switchpkg(None)
            return

        #Check to see if our packageType and name exists
        if not self._pkgs.has_key(self.parent.PackageType):
            self._switchpkg(None)
            return
        elif not self._pkgs[self.parent.PackageType].has_key(self.parent.PackageName):
            self._switchpkg(None)
            return
        else:
            check_dom = self._pkgs[self.parent.PackageType][self.parent.PackageName]

        #Check the Update
        #First Check the version and build numbers, if those are current, then we dont need to bother with a full check
        if check_dom.getAttribute('version') > self.parent.Version:
            updateAvaliable = True
        elif check_dom.getAttribute('version') == self.parent.Version and check_dom.getAttribute('build') > self.parent.Build:
            updateAvaliable = True

        if not updateAvaliable:
            self._showNoUpdate()
            return

        #Ok if we get here there is an update avaliable, now we do the file and directory check
        self._updateList['dirs'] = check_dom.getElementsByTagName('dir')
        self._updateList['files'] = []
        fs = check_dom.getElementsByTagName('file')
        self._progressbar.SetRange(len(fs))
        i = 0
        for file_dom in fs:
            checksum = md5.new()
            wx.Yield()
            self._progressbar.SetValue(i)
            i += 1
            name = file_dom.getAttribute('name')
            path = self.parent.PackageDirectory + os.sep + file_dom.getAttribute('path')
            fchecksum = file_dom.getAttribute('checksum')
            if not self.parent.verifyFile(path + os.sep + name):
                self._updateList['files'].append(file_dom)
                continue

            if file_dom.getAttribute('read_type') == 'rb':
                f = open(path + os.sep + name, "rb")
            else:
                f = open(path + os.sep + name, "r")
            data = f.read()
            checksum.update(data)
            f.close()

            if(checksum.hexdigest() != fchecksum):
                self._updateList['files'].append(file_dom)

        self._notes = check_dom.getAttribute('notes').replace(' ', '%20')
        self._showNotes()

    def _showNoUpdate(self, showSelect=True):
        self._hideAll()
        self._lastPage = 'NoUpdate'
        self.timer.Stop()
        self.parent.FindWindowById(wx.ID_FORWARD).SetLabel("Finish")
        self.parent.EnableButton(wx.ID_FORWARD)

        if showSelect:
            self._done.Show()
            self._pkgbtn.Show()
        else:
            self._notConected.Show()

        self.Layout()
        self.Next = None


    def _showNotes(self):
        self._hideAll()
        self._msgwnd.Show()
        self._noteswnd.Show()
        self._pkgbtn.Show()

        self._lastPage = 'Notes'

        f = urllib.urlopen(self._notes)
        self._noteswnd.SetPage(f.read())
        f.close()
        urllib.urlcleanup()

        dmsg = "A newer version is available.\n"
        for f in self._updateList['files']:
            dmsg += f.getAttribute('name') + " is out of date\n"
        dmsg += "Click on Next to Update now, or Calcel to skip this update.\n"
        dmsg += "It is recomended that you update now\n"
        self._msgwnd.SetValue(dmsg)

        self.Layout()

        self.Next = DownloadPage(self.parent, self._updateList, self._notes)
        wx.CallAfter(self.parent.EnableButton, wx.ID_CANCEL)
        wx.CallAfter(self.parent.EnableButton, wx.ID_FORWARD)

    def _showPackageSelect(self):
        self._hideAll()
        self._pkgsizer = wx.BoxSizer(wx.VERTICAL)
        rgroup = wx.RadioButton(self, wx.ID_ANY, "group_start", style=wx.RB_GROUP)
        rgroup.Hide()
        self._pkgsizer.Add(rgroup,0,0)
        for ptype, pkgs in self._pkgs.iteritems():
            self._pkgsizer.Add(wx.StaticText(self, wx.ID_ANY, ptype), 0, wx.EXPAND)

            for pkgname, dom in pkgs.iteritems():
                self._pkgsizer.Add(wx.RadioButton(self, wx.ID_ANY, pkgname, name=ptype + '^' + pkgname))

            self._pkgsizer.Add(wx.StaticLine(self))

        self._sizer.Add(self._pkgsizer, 1, wx.EXPAND)
        self.Layout()

        self.Next = None

        self.Bind(wx.EVT_RADIOBUTTON, self._pkgSet)

    def _switchpkg(self, evt):
        self._showPackageSelect()

    def _pkgSet(self, evt):
        if not self.timer.IsRunning():
            self.timer.Start(250)
        oldtype = self.parent.PackageType
        oldname = self.parent.PackageName

        bid = evt.GetId()
        btn = self.FindWindowById(bid)

        (ptype, name) = btn.GetName().split('^')

        if oldtype != ptype or oldname != name:
            self.parent.PackageType = ptype
            self.parent.PackageName = name
            self._checkUpdate(True)
        elif self._lastPage == 'Notes':
            self._showNotes()
        elif self._lastPage == 'NoUpdate':
            self._showNoUpdate()

    def _hideAll(self):
        self._progressbar.Hide()
        self._pkgbtn.Hide()
        self._msgwnd.Hide()
        self._noteswnd.Hide()
        self._notConected.Hide()
        self._done.Hide()
        try:
            for sitem in self._pkgsizer.GetChildren():
                if sitem.IsWindow():
                    wnd = sitem.GetWindow()
                    wnd.Hide()
                    wnd.Destroy()
            self._sizer.Remove(self._pkgsizer)
        except:
            pass

    def GetNext(self):
        return self._next

    def GetPrev(self):
        return self._prev

    def SetNext(self, page):
        self._next = page

    def SetPrev(self, page):
        self._prev = page

    Next = property(GetNext, SetNext)
    Prev = property(GetPrev, SetPrev)

class DownloadPage(wx.wizard.PyWizardPage):
    def __init__(self, parent, updateList, notes):
        wx.wizard.PyWizardPage.__init__(self, parent)
        self._updateList = updateList
        self._notes = notes
        self._next = None
        self._prev = None
        self.parent = parent

        self._sizer = wx.BoxSizer(wx.VERTICAL)
        self._msgsizer = wx.BoxSizer(wx.HORIZONTAL)

        self._noteswnd = NotesHTMLWindow(self)
        f = urllib.urlopen(self._notes)
        self._noteswnd.SetPage(f.read())
        f.close()
        urllib.urlcleanup()
        self._progressbar = wx.Gauge(self, wx.ID_ANY, 100)

        self._msgwnd = wx.TextCtrl(self, wx.ID_ANY, style=wx.TE_MULTILINE | wx.TE_READONLY)

        self._msgsizer.Add(self._noteswnd, 1, wx.EXPAND)
        self._msgsizer.Add(self._msgwnd, 1, wx.EXPAND)

        self._sizer.Add(self._msgsizer, 1, wx.EXPAND)
        self._sizer.Add(self._progressbar, 0, wx.EXPAND)

        self.SetSizer(self._sizer)
        self.SetAutoLayout(True)
        self.Layout()

        self.parent.DisableButton(wx.ID_CANCEL)
        self.parent.DisableButton(wx.ID_FORWARD)

        self._progressbar.SetRange(len(self._updateList['dirs']) + len(self._updateList['files']))

    def Start(self):
        wx.CallAfter(self.CreateDirs)

    def CreateDirs(self):
        self.parent.DisableButton(wx.ID_CANCEL)
        self.parent.DisableButton(wx.ID_FORWARD)

        i = 0
        for dir_dom in self._updateList['dirs']:
            self._progressbar.SetValue(i)
            i+= 1
            wx.Yield()
            path = self.parent.PackageDirectory + os.sep + dir_dom.getAttribute('path')
            if not self.parent.verifyFile(path):
                os.makedirs(path)
                self._msgwnd.Value = 'Created Directory: ' + path + "\n" + self._msgwnd.Value

        if i == 0:
            self._progressbar.SetValue(len(self._updateList['dirs']))

        wx.CallAfter(self.DownloadFiles)

    def DownloadFiles(self):
        self.parent.DisableButton(wx.ID_CANCEL)
        self.parent.DisableButton(wx.ID_FORWARD)

        if self.parent.Rollback:
            for root, dirs, files in os.walk(self.parent.PackageDirectory):
                for name in files:
                    if name.find('.rollback') != -1:
                        os.remove(root + os.sep + name)

        i = self._progressbar.GetValue()
        for file_dom in self._updateList['files']:
            self._progressbar.SetValue(i)
            i+= 1
            wx.Yield()
            name = file_dom.getAttribute('name')
            path = self.parent.PackageDirectory + os.sep + file_dom.getAttribute('path')
            url = file_dom.getAttribute('url').replace(' ', '%20')
            f = urllib.urlopen(self.parent.Url + url + '/' + name)
            data = f.read()
            f.close()
            urllib.urlcleanup()

            if self.parent.Rollback:
                if self.parent.verifyFile(path + os.sep + name + '.rollback'):
                    os.remove(path + os.sep + name + '.rollback')

                if self.parent.verifyFile(path + os.sep + name):
                    os.rename(path + os.sep + name, path + os.sep + name + '.rollback')

            if file_dom.getAttribute('read_type') == 'rb':
                f = open(path + os.sep + name, 'wb')
            else:
                f = open(path + os.sep + name, 'w')

            f.write(data)
            f.close()

            self._msgwnd.Value = 'Downloaded: ' + name + "\n" + self._msgwnd.Value

        if self._progressbar.GetValue() < self._progressbar.GetRange():
            self._progressbar.SetValue(self._progressbar.GetRange())

        wx.CallAfter(self.parent.EnableButton, wx.ID_FORWARD)

    def GetNext(self):
        return self._next

    def GetPrev(self):
        return self._prev

    def SetNext(self, page):
        self._next = page

    def SetPrev(self, page):
        self._prev = page

    Next = property(GetNext, SetNext)
    Prev = property(GetPrev, SetPrev)


class AutoUpdater(wx.wizard.Wizard):
    def __init__(self, *args, **kwargs):
        wx.wizard.Wizard.__init__(self, *args, **kwargs)

        self._url = None
        self._file = 'updates/package_list.xml'
        self._pkgDir = os.getcwd()
        self._pkgType = None
        self._pkgName = None
        self._version = 0
        self._build = 0
        self._rollback = False
        self.SetPageSize((640, 480))
        self.SetMinSize((640, 480))

        self.Bind(wx.wizard.EVT_WIZARD_FINISHED, self._onFinished)
        self.Bind(wx.wizard.EVT_WIZARD_CANCEL, self._onCanceled)
        self.FindWindowById(wx.ID_BACKWARD).Hide()

    def Setup(self, Url, File=None, PackageDirectory=None, PackageType=None, PackageName=None, Version=0, Build=0):
        self.Url = Url

        if not File is None:
            self.File = File

        if not PackageDirectory is None:
            self.PackageDirectory = PackageDirectory

        if not PackageType is None:
            self.PackageType = PackageType

        if not PackageName is None:
            self.PackageName = PackageName

        if Version != 0:
            self.Version = Version

        if Build != 0:
            self.Build = Build

    def Start(self):
        self.RunWizard(FirstPage(self))

    def DisableButton(self, bid):
        """
            Disable the button on the wizard with the given ID.
            Returns True if the button ID was found, false if not
        """
        try:
            btn = self.FindWindowById(bid)
        except Exception,e:
           return False
        # the enable needs to be "called after" as the wizard my re-enable
        #it due to the fact there is another page in that direction
        wx.CallAfter(btn.Disable)
        return True

    def EnableButton(self, bid):
        """
            Enable the button on the wizard with the given ID.
            Returns True if the button ID was found, false if not
        """
        try:
            btn = self.FindWindowById(bid)
        except Exception,e:
           return False
        # the enable needs to be "called after" as the wizard my re-enable
        #it due to the fact there is another page in that direction
        wx.CallAfter(btn.Enable)
        return True

    def verifyFile(self, path):
        """Returns True if file or directory exists"""
        try:
            os.stat(path)
            return True
        except OSError:
            return False

    #Callback Methods
    def Finished(self):
        pass

    #Event Methods
    def _onFinished(self, evt):
        self.Finished()
        wx.CallAfter(self.Destroy)

    def _onCanceled(self, evt):
        self.Destroy()

    #Property Methods
    def GetUrl(self):
        return self._url

    def SetUrl(self, url):
        self._url = url

    def GetUpdateFile(self):
        return self._file

    def SetUpdateFile(self, val):
        self._file = val

    def GetPackageDirectory(self):
        return self._pkgDir

    def SetPackageDirectory(self, val):
        self._pkgDir = val

    def GetPackageType(self):
        return self._pkgType

    def SetPackageType(self, ptype):
        self._pkgType = ptype

    def GetPackageName(self):
        return self._pkgName

    def SetPackageName(self, val):
        self._pkgName = val

    def GetVersion(self):
        return self._version

    def SetVersion(self, val):
        self._version = val

    def GetBuild(self):
        return self._build

    def SetBuild(self, val):
        self._build = val

    def GetRollback(self):
        return self._rollback

    def SetRollback(self, val):
        self._rollback = val

    Url                 = property(GetUrl,              SetUrl)
    UpdateFile          = property(GetUpdateFile,       SetUpdateFile)
    PackageDirectory    = property(GetPackageDirectory, SetPackageDirectory)
    PackageType         = property(GetPackageType,      SetPackageType)
    PackageName         = property(GetPackageName,      SetPackageName)
    Version             = property(GetVersion,          SetVersion)
    Build               = property(GetBuild,            SetBuild)
    Rollback            = property(GetRollback,         SetRollback)


class BlankwxApp(wx.App):
    def OnInit(self):
        wx.InitAllImageHandlers()
        self.frame = AutoUpdater(None, wx.ID_ANY, "OpenRPG Patcher")
        self.SetTopWindow(self.frame)
        self.frame.Url = 'http://openrpg.digitalxero.net/'
        self.frame.PackageDirectory = self.frame.PackageDirectory + os.sep + 'openrpg'
        self.frame.Start()
        return True

if __name__ == "__main__":
    app = BlankwxApp(0)
    app.MainLoop()











